/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package engcalculator;

import engcalculator.function.CalculusException;
import engcalculator.data.IntervalList;
import engcalculator.function.infix.InfixFunction;
import engcalculator.function.prefix.PrefixFunction;
import engcalculator.expression.Expression;
import engcalculator.expression.ParsingException;
import engcalculator.variable.Variable;
import java.util.LinkedList;

/**
 *
 * @author simone
 */
public class Calculator {
    static {
        try {
            PrefixFunction.initialize();
            InfixFunction.initialize();
            Variable.initialize();
        } catch (Exception e) {
            System.out.println (e.getMessage());
        }
    }

    private byte step=1;
    private final static byte MAXANSWER = 99;
    private Expression expression;
    
    public String compute (String expressionString) {
        expression=null;
        Variable answer;
        IntervalList result;
        StringBuffer ans = new StringBuffer ();
        
        try {
            expression  = new Expression (expressionString, null);
            result = expression.evaluate(null);

            LinkedList<String> dup = expression.listOfDuplicatedIntervals();
            if (dup.size() != 0) {
                ans.append("Warning: there may be duplicated variables in the input expression which may lead to overstimation of the result. Try to rewrite the input expression avoiding the following duplications for a smaller result: ");
                ans.append(dup);
                ans.append("\n\n");
            }

            if (result == null) {
                return "No answer returned";
            } else if (result.getName() != null || (result.size() == 1 && result.getFirst().getName() != null)) {
                ans.append(result.toString());
            } else if (result.size() == 0) {
                ans.append("No answer returned");
            } else {
                Variable.addVariable(new Variable ("lastanswer", result,  false, "It contains last answer provided by the parser."));
                answer = new Variable ("answer"+step%MAXANSWER, result,  false, "");
                ++step;
                Variable.addVariable(answer);
                ans.append(answer.getValue().toString());
            }
            return ans.toString();

        } catch (CalculusException ce) {
            return "There was an error tring to execute the input expression.\n"+ce.getMessage();
        } catch (ParsingException pe) {
            return "There was an error tring to parse the input expression.\n"+pe.getMessage();
        } catch (Exception ex) {
            ex.printStackTrace(System.out);
            return "There was an internal error\nPlease send the current expression and error to the author for debug:\n"+expression.toString()+"\n"+ex.getMessage();
        }

    }

    public int getStep() {
        return step;
    }

    void stopCalculus() {
        if (expression != null) expression.stopCalculus();
    }

}
