/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package engcalculator.data;

import engcalculator.expression.ParsingException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;

/**
 *
 * @author simone
 */
public class Interval {
    public static enum WorkFlow {NEXT_SUBEX, PREVIOUS_SUBEX, THIS_SUBEX, NEXT_SUBEX_THEN_THIS_SUBEX, STOP, SKIP_NEXT_SUBEX};


    public final static String ENGUNITS = "yzafpnumKMGTPEZY";
    public final static String NUMBNOTATION = "0123456789.e";    
    private final static String UNITS = "yzafpnum KMGTPEZY";
    private final static boolean DIRECTED_INTERVAL = true;
    private static boolean avoidOuterExtimation = true;

    private static DecimalFormat dfVal = new DecimalFormat ("####.###", new DecimalFormatSymbols (Locale.ENGLISH));

    private static boolean valTol = true;
    private static boolean engFormat = true;
    private static int numberWidth = 15;

    public final static void setValTolFormat (boolean printValTol, boolean printEngFormat, int valDigits, int width) {
        valTol = printValTol;
        engFormat = printEngFormat;

        StringBuffer val = new StringBuffer ("####.");
        while (valDigits -- > 0) val.append('#');
        dfVal = new DecimalFormat (val.toString(), new DecimalFormatSymbols (Locale.ENGLISH));

        numberWidth = width;
    }

    public final static Interval valueOf (String engnotation) throws ParsingException {
        final int length = engnotation.length();

        int nextIndex, currentIndex;
        char currChar;
        nextIndex = currentIndex = 0;

        //skip the initial sign
        currChar = engnotation.charAt(0);
        if (currChar == '-' || currChar == '+') {
            currentIndex = 1;
        }

        //look for the value
        while (currentIndex < length && (Character.isDigit(currChar = engnotation.charAt(currentIndex)) || ".e-".indexOf(currChar) != -1)) {
            ++currentIndex;
        }

        //convert the value
        double val = Double.parseDouble(engnotation.substring(nextIndex,currentIndex));

        //look if a engineer coefficient is present and get the power
        if (currentIndex < length) {
            int power = UNITS.indexOf(engnotation.charAt(currentIndex));
            if (power == -1)
                power = 8;
            else
                ++currentIndex;

            //apply the power
            val *= Math.pow(10, 3*(power-8));
        }

        if (currentIndex < length) throw new ParsingException ("It was expected the end of the interval.");
        return new Interval(val);
    }

    public static void setAvoidOuterExtimation(boolean avoidOuterExtimation) {
        Interval.avoidOuterExtimation = avoidOuterExtimation;
    }

    private double left, right;

    public Interval (double val) {
        left = right = val;
    }

    public Interval (Interval i) {
        this (i.left, i.right);
    }

    public Interval (double left, double right) {
        if (DIRECTED_INTERVAL || left <= right) {
            this.left = left;
            this.right = right;
        } else {
            this.left = right;
            this.right = left;
        }
    }

    public Interval setOuter () {
        if (avoidOuterExtimation) return this;

        if (isScalar()) return this; //that avoids to expand a scalar

        if (! (Double.isInfinite(left) || Double.isNaN(left)) ) {
            if (left == 0) left = -Double.longBitsToDouble(1);
            else if (left > 0) left = Double.longBitsToDouble(Double.doubleToLongBits(left)-1);
            else left = Double.longBitsToDouble(Double.doubleToLongBits(left)+1);
        }

        if (! (Double.isInfinite(right) || Double.isNaN(right)) ) {
            if (right == 0) right = Double.longBitsToDouble(1);
            else if (right > 0) right = Double.longBitsToDouble(Double.doubleToLongBits(right)+1);
            else right = Double.longBitsToDouble(Double.doubleToLongBits(right)-1);
        }        

        return this;
    }

    public double getRight() throws Exception {
        return right;
    }

    public double getLeft() throws Exception {
        return left;
    }

    public double getTol () {
        return (right-left)*100/Math.abs(right+left);
    }

    public double getVal () {
        if (isScalar()) return left; //to avoid error propagation
        return (right+left)/2;
    }

    public double getRange () {
        return right-left;
    }
    
    public boolean isScalar () {
        return left == right;
    }

    public boolean isInteger () {
        return isScalar() && left == Math.floor(left);
    }

    public boolean isEven () {
        return isInteger() && (((int) left)&1)==0;//left%2 == 0;
    }

    public boolean isOdd () {
        return isInteger() && (((int) left)&1)!=0;//left%2 != 0;
    }

    public boolean isPositive () {
        return left >= 0 && right >= 0;
    }

    public boolean isNegative () {
        return left <= 0 && right <= 0;
    }

    public boolean isProper () {
        return left <= right;
    }

    public boolean containsZero () {
        return (left <= 0 && 0 <= right) || (right <= 0 && 0 <= left);
    }

    public String getName () {
        return null;
    }

    private StringBuffer toEngString (double a) {
        double olda = a;
        int coeff = 0;
        while ( Math.abs(a) < 1 && coeff >= -8) {
            a *= 1000;
            coeff -= 1;
        }
        while ( Math.abs(a) >= 1000 && coeff <= 8) {
            a /= 1000;
            coeff += 1;
        }

        StringBuffer result = new StringBuffer ();
        if (coeff >= 8 || coeff <= -8) {
            coeff = 0;
            a = olda;
            if (a == 0.0) result.append('0');
            else {
                String sval = Double.toString(a);
                sval = sval.replace('E', 'e');
                result.append(sval);
            }
        } else result.append(dfVal.format(a));
        if (coeff!=0) result.append(UNITS.charAt(coeff+8));
        return result;
    }

    @Override
    public String toString () {
        StringBuffer result = new StringBuffer();
        if (valTol) {
            if (engFormat) {
                result.append(toEngString(getVal()));
                double t = getTol();
                if (Math.abs(t)>1e-6) result.append('%').append(toEngString(t));
            } else {
                result.append(getVal()).append('%').append(getTol());
            }
        } else {
            double m, M;
            try {
                m = getLeft();
            } catch (Exception e) {
                m = Double.NaN;
            }
            try {
                M = getRight();
            } catch (Exception e) {
                M = Double.NaN;
            }
            if (engFormat) {
                result.append(toEngString(m)).append('_').append(toEngString(M));
            } else {
                result.append(m).append('_').append(M);
            }
        }

        return result.toString();
    }

    protected String toStringSpaced () {
        StringBuffer sb = new StringBuffer (numberWidth);
        final String s = toString();
        final int spaces = numberWidth - s.length();
        for (int i = 0; i < spaces; ++ i) sb.append(' ');
        sb.append (s);
        return sb.toString();
    }

    public WorkFlow getWorkFlow () {
        return WorkFlow.NEXT_SUBEX;
    }
}
