/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package engcalculator.expression;

import engcalculator.expression.token.*;
import engcalculator.function.prefix.PrefixFunction;
import java.util.HashMap;
import java.util.LinkedList;

/**
 *
 * @author simone
 */
public class Tokenizer { //just a container for all types of components findable
    private static final boolean DEBUG = false;
    
    private final LinkedList<Token> tokens;
    private HashMap <String, PrefixFunction> parameters;
    private LinkedList <String> parametersName;
    
    public Tokenizer () {//Warning the order of insertion is list is also the priority order for token recognizing
        tokens = new LinkedList<Token> ();

        tokens.add(new IntervalTkn());
        tokens.add(new VariableTkn());
        tokens.add(new ParameterTkn(this));
        tokens.add(new PrefixFunctionTkn());
        tokens.add(new InfixFunctionTkn());
        tokens.add(new LiteralTkn());
        tokens.add(new TerminalTkn());
        tokens.add(new SpaceTkn());        
    }

    public HashMap<String, PrefixFunction>  getParameters() {
        return parameters;
    }

    public LinkedList<String> getParametersName() {
        return parametersName;
    }

    public void setParameters(HashMap<String, PrefixFunction> parameters) {
        this.parameters = parameters;
    }

    // Create the list of tokens required to interpeter the expression
    public TokenList tokenize (LinkedList <String> parametersName, String expression) throws Exception {
        this.parametersName = parametersName;
        Token.setDefiningFunction (parametersName != null);


        TokenList tokenizedExpression = new TokenList ();
        String exp = new String(expression);
        Token token;
        String tokenSymbol;
        int index;
        boolean found;

        while (exp.length() > 0) {

            index = 0;
            found = false;
            while (++index <= exp.length()) {
                found = false;
                token = null;
                tokenSymbol = exp.substring(0, index);
                for (Token t : tokens)
                    if (t.mayBeCompatible(tokenSymbol)) {
                        found = true;
                        break;
                    }
                if (! found) break; //it was not found any item compatible
            }
            if ( !found && index == 1) throw new ParsingException("It is not possible find any token at "+exp);

            do {
                --index;
                tokenSymbol = exp.substring(0, index);

                token = null;
                for (Token t : tokens)
                    if (t.mayBeCompatible(tokenSymbol)) {
                        token = t.generateToken(tokenSymbol);
                        if (token != null) {
                            if (token.isSignificant()) {
                                tokenizedExpression.add(token);
                            }
                            break;
                        }
                    }
            } while (token == null && index > 1);
            if (token == null) throw new ParsingException("It is not possible recognize the token at "+exp);
            exp = exp.substring(index);

        }

        if (DEBUG) System.out.println ("Tokenized: "+tokenizedExpression.toString());

        return tokenizedExpression;
    }

    public static void retokenize (TokenList tl) {
        final int s = tl.size();
        for (int i=0; i<s; ++i)
            tl.set(i, tl.get(i).regenerateToken());

        if (DEBUG) System.out.println ("Retokenized: "+tl.toString());
    }
}
