/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package engcalculator.function;

import engcalculator.data.IntervalList;
import engcalculator.function.infix.InfixFunction;
import engcalculator.function.prefix.PrefixFunction;
import engcalculator.variable.Variable;
import java.util.LinkedList;

/**
 *
 * @author simone
 */
public abstract class Function {

    public static LinkedList<Function> getFunctions () {
        LinkedList<Function> result = new LinkedList<Function> ();
        result.addAll(PrefixFunction.getPrefixFunctions());
        result.addAll(InfixFunction.getInfixFunctions());
        result.addAll(Variable.getVariables());
        return result;
    }

    public static LinkedList<String> getFunctionSymbols () {
        LinkedList<String> result = new LinkedList<String> ();

        for (PrefixFunction f : PrefixFunction.getPrefixFunctions()) 
            result.add(new StringBuffer(f.getSymbol()).append('(').toString());
        
        for (InfixFunction f : InfixFunction.getInfixFunctions())
            result.add(f.getSymbol());

        for (Variable v : Variable.getVariables())
            result.add(v.getSymbol());

        return result;
    }

    public static String checkFunctions () {
        StringBuffer result = new StringBuffer ();
        String check;
        for (Function pf : getFunctions ()) {
            if ((check = pf.selfCheck()) != null) result.append("\n**** FAILED ").append("Function ").append(pf.getSymbol()).append(" :  ").append(check);
            else result.append ("\nPASSED ").append("Function ").append(pf.getSymbol());
        }
        return result.toString();
    }


    private String symbol;          //it is the symbol to be find reading a text
    private String help;            //the help showed for the variable
    private boolean constant;       //if the funcion is constant it cannot be overwrittent neither deleted

    public Function(String symbol, boolean constant, String help) {
        this.symbol = symbol;
        this.help = help;
        this.constant = constant;
    }

    public String getSymbol() {
        return symbol;
    }

    @Override
    public final String toString() { 
        return symbol;
    }

    public String getHelp() {
        return help;
    }

    public boolean isConstant() {
        return constant;
    }

    protected abstract String selfCheck ();

    protected String removeSpaces (String input) {
        StringBuffer output = new StringBuffer ();
        for (char c : input.toCharArray())
            if (c != ' ') output.append (c);
        return output.toString();
    }

    public abstract int getNumberOfArguments ();

    public abstract IntervalList compute (IntervalList... arguments) throws Exception;

}
