/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package engcalculator.function.prefix.equation;


import engcalculator.data.Interval;
import engcalculator.data.IntervalList;
import engcalculator.data.MatrixIntervalList;
import engcalculator.function.CalculusException;
import engcalculator.function.prefix.PrefixFunction;
import engcalculator.function.infix.arithmetic.Div;
import engcalculator.function.infix.compare.Greater;
import engcalculator.function.infix.arithmetic.Mul;
import engcalculator.function.infix.arithmetic.Sub;
import engcalculator.function.prefix.interval.Dual;

/**
 *
 * @author simONE
 */
public class GaussElimination extends PrefixFunction {
    private final static String[] EXAMPLE = {"(1, 2, 3, 4)#2,(5,6)#1"};
    private final static String[] RESULT = {"(-4,4.5)#1"};
    private final static Greater GREATER = new Greater();
    private final static Mul MUL = new Mul();
    private final static Div DIV = new Div ();
    private final static Sub SUB = new Sub();
    private final static Dual DUAL = new Dual();


    public GaussElimination() {
        super("gausselimination", (byte) -1, false, true, "Diagonalize a matrix using Gauss elimination algorithm to solve simultaneous equation. The source code was adapted from http://snippets.dzone.com/posts/show/4874 It is still not properly working on intervals, I have to understand if it is possible to solve simultaneous equations with intervals...", EXAMPLE, RESULT);
    }


    @Override
    public IntervalList _compute(IntervalList input) throws Exception {
        if (! input.isProperMatrix() && input.getRows() != input.getColumns()-1) throw new CalculusException ("The input matrix for a system should have n rows and n+1 columns. ");
        MatrixIntervalList result = new MatrixIntervalList (input);
        forwardSubstitution(result);
        reverseElimination(result);
        return getSolutionColumn(result);

    }

    void forwardSubstitution(MatrixIntervalList result) throws Exception {
        int i, j, k, max;
        final int n=result.getRows();
        Interval t;
        for (i = 0; i < n; ++i) {
            max = i;
            for (j = i + 1; j < n; ++j)
                if (GREATER.compare(result.get(j, i), result.get(max, i)))
                    max = j;

            for (j = 0; j < n + 1; ++j) {
                t = result.get(max,j);
                result.set(max,j, result.get(i,j));
                result.set(i, j, t);
            }

            for (j = n; j >= i; --j)
                for (k = i + 1; k < n; ++k) {
                    result.set(k, j, SUB._computeIntervals(result.get(k, j), DUAL._computeIntervals(DIV._computeIntervals(MUL._computeIntervals(result.get(k, i), result.get(i, j)), DUAL._computeIntervals(result.get(i ,i))))));
                }

        }
    }

    void reverseElimination(MatrixIntervalList result) throws Exception {
        int i, j;
        final int n=result.getRows();
        for (i = n - 1; i >= 0; --i) {
            result.set(i, n, DIV._computeIntervals(result.get(i, n), DUAL._computeIntervals(result.get(i, i))));
            for (j = i - 1; j >= 0; --j) {
                result.set(j, n, SUB._computeIntervals(result.get(j, n), DUAL._computeIntervals(MUL._computeIntervals(result.get(j,i), result.get(i, n)))));
            }
        }
    }

    private MatrixIntervalList getSolutionColumn(MatrixIntervalList result) {
        MatrixIntervalList output = new MatrixIntervalList(1);
        final int col = result.getColumns()-1, row = result.getRows();
        for(int r=0;r<row;++r) {
            output.add(result.get(r,col));

        }
        return output;
    }

}
